import os
import json


class Credentials:

    def __init__(self, config_file: str = 'snowflake_credentials.json',
                 conn_name: str = '') -> None:
        """Instantiates instances of the needed params to locate creds file.

        Args:
            config_file: Name of .json configuration file following the
            format of connection_credentials_SAMPLE.json.
            conn_name: Name of connection within json file to use - it will
            use first set of credentials in the file if no argument is passed.

        """

        self.config_file = config_file
        self.conn_name = conn_name

    def get(self) -> dict:
        """Locates creds file and parses out the specified set of credentials.

        Returns:
            Dictionary containing a specific set of Snowflake credentials

        """
        print("Locating & importing credentials..")
        print(f"\t<1 of 4> Searching for {self.config_file} in local file "
              f"system..")
        for dirpath, dirnames, files in os.walk(os.path.expanduser('~'),
                                                topdown=False):
            if self.config_file in files:
                path_to_config = os.path.join(dirpath, self.config_file)
                break
            else:
                path_to_config = ''
                pass

        if not os.path.isfile(path_to_config):
            print(f"\tCould not find {self.config_file} in file system!")
        else:
            with open(path_to_config) as c:
                all_creds = json.load(c)
            print(f"\n\t<2 of 4> Located & loaded {self.config_file} "
                  f"from:\n\t\t{path_to_config}")

        if not self.conn_name:
            self.conn_name = next(iter(all_creds.keys()))
            print(f"\n\t<3 of 4> No explicit connection passed, fetching "
                  f"'{self.conn_name}' credentials by default")
        else:
            print(f"\n\t<3 of 4> Fetching '{self.conn_name}' credentials "
                  f"from {self.config_file}")
            # pass

        creds = all_creds.get(self.conn_name)

        if creds:
            print(f"\n\t<4 of 4> Successfully imported credentials for "
                  f"conn_name='{self.conn_name}'")

        else:
            print(f"\n\t<4 of 4> Could not parse conn_name='{self.conn_name}'"
                  f"from {self.config_file}\n\t\tPlease specify a different "
                  f"configuration file, connection name, or check that the "
                  f"contents of the configuration file")

        return creds

# creds = Credentials().get()
# creds.get()
# creds.conn_name



from snowmobile import snowconn
from snowmobile import snowcreds
from snowmobile import snowquery
from snowmobile import snowloader
from snowmobile import snowscripter

if __name__ == "__main__":
    main()



import re
import sqlparse
from IPython.core.display import display, Markdown
from snowmobile import snowquery
import os


class Statement:

    def __init__(self, sql, snowflake: snowquery.Connector = ''):

        self.sql = sql
        self.snowflake = snowflake

    def render(self) -> object:
        """Renders SQL as markdown when called in IPython environment.

        Useful when combining explanation of different components of a SQL
        script in IPython or similar environment.

        """
        if isinstance(self.sql, list):
            self.sql = ';\n\n'.join(self.sql)

        display(Markdown(f"```mysql\n{self.sql}\n```"))

        return None

    def raw(self) -> object:
        """Returns raw SQL text."""

        return self.sql

    def execute(self, return_results=True, render=False,
                include_description=False) -> object:
        """Executes sql with option to return results / render sql as Markdown.
        """
        self.results = self.snowflake.execute_query(self.sql)

        if render:
            self.render()

        if include_description:
            print(f"\n{self.results.describe()}\n")

        if return_results:
            return self.results


class Script(Statement):
    """
    Active-parsing of SQL script in Python for more seamless development loop.

    Enables flexible parsing of a SQL script into individual statements or
    spans of statements based on statement headers declared in SQL script.
    These should be denoted by placing headers above the statements within
    within the SQL file as shown below:

    .. code-block:: sql

        /*-statement_header-*/
        create or replace table sample_table as
        select
            *
        from sample_other

    """

    def __init__(self, path: str, pattern: str = r"/\*-(\w.*)-\*/",
                 snowflake: snowquery.Connector = ''):
        """Instantiating an instance of 'script' by calling Script class
        on a path to a SQL script.

        Structured this way so that each method within the class
        re-instantiates a new instance of 'script' (i.e. re-imports the raw
        file from local).

        This keeps users from having to alter or re-import anything on the
        Python side while editing & saving changes to the SQL file as well as
        ensures that all methods are using the latest version of the script.

        Args:
            path: Full path to SQL script including .sql extension
            pattern: Regex pattern that SQL statement headers are wrapped in
            snowflake: Instantiated snowquery.Connector instance to use in
            the execution of Script or Statement objects

        Instantiated Attributes:
            script_txt:
                (str) raw SQL script read in as text file
            list_of_statements:
                (list) list of all SQL statements in SQL script (both with and
                without headers)
            statement_names:
                (list) List of statement headers - including blank entries for
                statements that do not have headers (these instances have
                entries equal to '[]')
            statement:
                (dict) Dictionary containing {'header': 'associated_sql'} - note
                the conditionals within the comprehensions that throw out entries
                from both lists that do not have a valid statement_header
            spans:
                (dict) Dictionary containing {'header': 'index_position_in_script'}
                and is used in the get_span() method
            ordered_statements:
                (list) List of statements in the order in which they appear in the
                script. Note that these are only statements that have a valid
                headers in the SQL file
            header_statements:
                (str) List of all statements with their headers prepended to them,
                same length as ordered_statements
            full_sql:
                (str) String containing all statements with valid headers combined
                into a single string.

        """
        super().__init__(self)
        self.pattern = pattern
        self.source = path
        self.snowflake = snowflake
        self.name = os.path.split(self.source)[-1].split('.sql')[0]
        self.pattern = re.compile(pattern)
        with open(self.source, 'r') as f:
            self.script_txt = f.read()
        self.list_of_statements = sqlparse.split(self.script_txt)
        self.statement_names = \
            [self.pattern.findall(self.statement) for
             self.statement in self.list_of_statements]

        self.statements = {
            k[0].lower():
                sqlparse.format(v, strip_comments=True).lstrip().rstrip()
            for k, v in zip(self.statement_names, self.list_of_statements)
            if k != []
        }

        self.spans = {val: i for i, val in enumerate(self.statements.keys())}
        self.ordered_statements = [val for val in self.statements.values()]
        self.header_statements = [f"/*-{head.upper()}-*/\n{sql}"
                                  for head, sql in self.statements.items()]
        self.full_sql = ";\n\n".join(self.header_statements)
        self.returned = {}

    def reload_source(self) -> object:
        """Reloads script from source file path.

        """
        with open(self.source, 'r') as f:
            self.script_txt = f.read()

        return self

    def run(self) -> None:
        """Executes entire script a statement at a time."
        """
        self.reload_source()

        if not self.snowflake:
            self.snowflake = snowquery.Connector()

        for raw_sql in self.list_of_statements:
            self.snowflake.execute_query(raw_sql)

    def get_statements(self) -> object:
        """Gets dictionary of unique Statement objects & associated methods.

        Returns:
            Dictionary of unique Statement objects & following methods:

        .. code-block:: python

            statement.render()
            statement.run()
            statement.execute()
        """
        self.reload_source()
        self.statements = Script(self.source).statements
        for k, v in self.statements.items():
            self.returned[k] = Statement(v, self.snowflake)

        return self.returned

    def fetch(self, header) -> object:
        """Fetches a single statement object out of the script.

        Most convenient method to execute individual statements - use
        get_statements() method if wanting to iterate over multiple
        statement objects and access the same methods

        Args:
            header: Header/label of the statement to fetch

        Returns:
            Statement object on which the following methods can be called:

        .. code-block:: python

            statement.render()
            statement.run()
            statement.execute()
        """
        return self.get_statements().get(header)



import pathlib
from setuptools import setup
from setuptools import find_packages
import subprocess

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


version_str = \
    str((subprocess.check_output(['git', 'describe']).strip())).\
        split('-')[0].replace("'", '').replace('b', '')

# This call to setup() does all the work
setup(
    name="snowmobile",
    version=version_str,
    description="A simple set of modules for streamlined interaction with the Snowflake Database",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/GEM7318/Snowmobile",
    author="Grant Murray",
    author_email="gmurray203@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    # packages=["snowmobile"],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=['pandas', 'snowflake-connector-python', 'sqlparse'],
    entry_points={
        "console_scripts": [
            "snowmobile=snowmobile.__main__:main",
        ]
    },
)




import snowflake.connector as sf
import pandas as pd
from snowmobile import snowconn


class Connector(snowconn.Connection):
    """Primary Connection and Query Execution Class"""

    def __init__(self, config_file: str = 'snowflake_credentials.json',
                 conn_name: str = '') -> None:
        super().__init__(config_file, conn_name)

        self.conn = snowconn.Connection(config_file=self.config_file,
                                        conn_name=self.conn_name).get_conn()

    def execute_query(self, query: str, from_file: bool = False,
                      filepath='', return_results=True) -> pd.DataFrame:
        """Run commands in Snowflake.

        Args:
            query: Raw SQL to execute.
            from_file: Boolean value indicating whether or not to read the
            query from a local file.
            filepath: Full file path to .sql script if looking to execute a
            single-statement script on the fly, most useful for executing
            extracted DDL as part of a cleanup.
        Returns:
            Results from query in a Pandas DataFrame by default or None if
            ``return_results=False`` is passed when function is called.
        """
        self.query = query
        self.from_file = from_file
        self.filepath = filepath
        self.return_results = return_results

        if self.from_file and self.filepath:

            try:
                self.query = open(self.filepath).read()

            except sf.errors.ProgrammingError as e:
                print("There was an error reading the file.")
                self.query = None

        if self.query:

            try:
                self.results = pd.read_sql(query, self.conn)

            except sf.errors.ProgrammingError as e:
                print(e)  # default error message
                print(f'Error {e.errno} ({e.sqlstate}): {e.msg} ('
                      f'{e.sfqid})')  # custom error message
                self.results = None

        else:
            self.results = None

        if self.return_results:
            return self.results

        else:
            return None

    def disconnect(self) -> None:
        """Disconnect from connection with which Connect() was instantiated.

        Returns:
            None

        """
        self.conn.close()
        return None
